;;; lanna.el --- Quail package to input Tai Tham characters -*-coding: utf-8;-*-

;; Copyright (C) 2013
;; Richard Wordingham

;; This file is intended for use with GNU Emacs.

;; This file is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; It is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
;; Commentary

;; This defines one keyboard mapping, based on the Thai kesmanee keyboard,
;; for the Tai Tham script.

;; Code:

(require 'quail)

(quail-define-package
 "lanna" "Northern Thai" "ᩓ" nil
 "Kesmanee-based keyboard map"
 nil t t t t nil nil nil nil nil t)

(quail-define-rules
 ("`" ?๏)       ; U+0E4F THAI CHARACTER FONGMAN
 ("1" ?\u1a64)  ; U+1A64 TAI THAM VOWEL SIGN TALL AA
 ("2"  ?/)      ; U+002F SOLIDUS
 ("!2" ?\u1aa8) ; U+1AA8 TAI THAM SIGN KAAN
 ("3" ?-)
 ("4" ?ᨽ)       ; U+1A3D TAI THAM LETTER LOW PHA
 ("5" ?ᨳ)       ; U+1A33 TAI THAM LETTER HIGH THA
 ("6" ?\u1a69)  ; U+1A69 TAI THAM VOWEL SIGN U
 ("!6" ?\ᩏ)     ; U+1A4F TAI THAM LETTER U
 ("7" ?\u1a67)  ; U+1A67 TAI THAM VOWEL SIGN UE
 ("8" ?ᨣ)       ; U+1A23 TAI THAM LETTER LOW KA
 ("9" ?ᨲ)       ; U+1A32 TAI THAM LETTER HIGH TA
 ("!9" ?\ufe02) ; U+FE02 VARIATION SELECTOR-3 - illicit compact flag
 ("0" ?ᨧ)       ; U+1A27 TAI THAM LETTER HIGH CA
 ("-" ?ᨡ)       ; U+1A21 TAI THAM LETTER HIGH KHA
 ("!-" ?\ufe01) ; U+FE01 VARIATION SELECTOR-2 - illicit ascender flag
 ("=" ?ᨩ)       ; U+1A29 TAI THAM LETTER LOW CA
 ("!=" ["ᩉ᩠ᩃ"]) ; U+1A49, U+1A60, U+1A43

 ("¬" ?๛)     ; U+0E5B THAI CHARACTER KHOMUT
 ("!!" ?+)     ; U+002B PLUS SIGN
 ("\"" ?᪁)      ; U+1A81;TAI THAM HORA DIGIT ONE
 ("!\"" ?᪑)     ; U+1A91;TAI THAM THAM DIGIT ONE
 ("£"  ?᪂)      ; U+1A82;TAI THAM HORA DIGIT TWO
 ("!£" ?᪒)      ; U+1A92;TAI THAM THAM DIGIT TWO
 ("$"  ?᪃)     ; U+1A83;TAI THAM HORA DIGIT THREE
 ("!$" ?᪓)     ; U+1A93;TAI THAM THAM DIGIT THREE
 ("%"  ?᪄)     ; U+1A84;TAI THAM HORA DIGIT FOUR
 ("!%" ?᪔)     ; U+1A94;TAI THAM THAM DIGIT FOUR
 ("^" ?\u1a6a) ; U+1A6A TAI THAM VOWEL SIGN UU
 ("!^" ?ᩐ)     ; U+1A50 TAI THAM LETTER UU
; ("&" ?)    ; U+
 ("*"  ?᪅)     ; U+1A85;TAI THAM HORA DIGIT FIVE
 ("!*" ?᪕)     ; U+1A95;TAI THAM THAM DIGIT FIVE
 ("("  ?᪆)     ; U+1A86;TAI THAM HORA DIGIT SIX
 ("!(" ?᪖)     ; U+1A96;TAI THAM THAM DIGIT SIX
 (")"  ?᪇)     ; U+1A87;TAI THAM HORA DIGIT SEVEN
 ("!)" ?᪗)    ; U+1A97;TAI THAM THAM DIGIT SEVEN
 ("_"  ?᪈)    ; U+1A88;TAI THAM HORA DIGIT EIGHT
 ("!_" ?᪘)    ; U+1A98;TAI THAM THAM DIGIT EIGHT
 ("+"  ?᪉)    ; U+1A89;TAI THAM HORA DIGIT NINE
 ("!+" ?᪙)    ; U+1A99;TAI THAM THAM DIGIT NINE

 ("q"  ?\u1a7b) ; U+1A7B TAI THAM SIGN MAI SAM
 ("!q" ?\u1aa7) ; U+1AA7 TAI THAM SIGN MAI YAMOK
 ("Q"  ?᪀)      ; U+1A80 TAI THAM HORA DIGIT ZERO
 ("!Q" ?᪐)      ; U+1A90 TAI THAM THAM DIGIT ZERO
 ("w"  ?\u1a71) ; U+1A71 TAI THAM VOWEL SIGN AI
 ("!w" ?\u034F) ; U+034F COMBINING GRAPHEME JOINER
 ("W"  ?\")     ; U+0022 QUOTATION MARK
 ("!W" ?\u1aa9) ; U+1AA9 TAI THAM SIGN KAANKUU
 ("e"  ["\u1a63\u1a74"])
 ("!e" ["\u1a64\u1a74"]) ; U+
 ("E"  ?\u1a60) ; U+1A60 TAI THAM SIGN SAKOT
 ("r"  ?ᨻ)      ; U+1A3B TAI THAM LETTER LOW PA
 ("!r" ?\u1a5b) ; U+1A5B TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA
 ("R"  ?\u1a5b) ; U+1A5B TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA
 ("!R" ?\u1a5a) ; U+1A5A;TAI THAM CONSONANT SIGN LOW PA
 ("t"  ?\u1a61) ; U+1A61 TAI THAM VOWEL SIGN A
 ("T" ?ᨵ)       ; U+1A35;TAI THAM LETTER LOW THA
 ("y"  ?\u1a62) ; U+1A62 TAI THAM VOWEL SIGN MAI SAT
 ("Y"  ?\u1a74) ; U+1A74 TAI THAM SIGN MAI KANG
 ("u"  ?\u1a66) ; U+1A66 TAI THAM VOWEL SIGN II
 ("!u" ?\u1a4e) ; U+1A4E TAI THAM LETTER II
; ("U"  ?) ; U+
 ("i"  ?ᩁ)      ; U+1A41 TAI THAM LETTER RA
 ("!i" ?ᩕ)       ; U+1A55 TAI THAM CONSONANT SIGN MEDIAL RA
 ("I"  ?ᨱ)      ; U+1A31 TAI THAM LETTER RANA
 ("!I" ["ᩉᩕ"])      ; U+1A49 U+1A55
 ("o"  ?ᨶ)      ; U+1A36 TAI THAM LETTER NA
 ("!o" ["ᩉ᩠ᨶ"]) ; U+1A49, U+1A60, U+1A36
 ("O"  ?ฯ)      ; U+1AAA TAI THAM SIGN SATKAAN
 ("!O" ?\u1aab) ; U+1AAB;TAI THAM SIGN SATKAANKUU
 ("p"  ?ᨿ)      ; U+1A3F TAI THAM LETTER LOW YA
 ("!p" ?ᩀ)      ; U+1A40 TAI THAM LETTER HIGH YA
 ("P"  ?ᨬ)      ; U+1A2C TAI THAM LETTER NYA
 ("!P" ["ᩉ᩠ᨿ"]) ; U+1A49 U+1A60 U+1A3F
 ("["  ?ᨷ)      ; U+1A37 TAI THAM LETTER BA
 ("![" ?\u1a5d) ; U+1A5D TAI THAM CONSONANT SIGN BA
 ("{"  ?ᨮ)      ; U+1A2ETAI THAM LETTER HIGH RATHA
 ("!{" ?\u1a5b) ; U+1A5B TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA
 ("]"  ?ᩃ)      ; U+1A43 TAI THAM LETTER LA
 ("!]" ?\u1a56) ; U+1A56 TAI THAM CONSONANT SIGN MEDIAL LA
 ("}"  ?,)      ; U+002C COMMA
 ("!}" ["ᩉᩖ"])  ; U+1A49 1A56

 ("a"  ?ᨼ)      ; U+1A3C TAI THAM LETTER LOW FA
 ("A"  ?ᩂ)      ; U+1A42 TAI THAM LETTER RUE
 ("s"  ?ᩉ)      ; U+1A49 TAI THAM LETTER HIGH HA
 ("!s" ?\u1a73) ; U+1A73 TAI THAM VOWEL SIGN OA ABOVE
 ("S"  ?ᨥ)      ; U+1A25 TAI THAM LETTER LOW KHA
 ("d"  ?ᨠ)      ; U+1A20 TAI THAM LETTER HIGH KA
 ("D"  ?ᨭ)      ; U+1A2D TAI THAM LETTER RATA
 ("f"  ?ᨯ)      ; U+1A2F TAI THAM LETTER DA
 ("!f" ?\u1a77) ; U+1A77 TAI THAM SIGN KHUEN TONE-3
 ("F"  ?\u1a70) ; U+1A70 TAI THAM VOWEL SIGN OO
 ("!F" ?\u1a52) ; U+1A52 TAI THAM LETTER OO
 ("g"  ?\u1a6e) ; U+1A6E TAI THAM VOWEL SIGN E
 ("!g" ?ᩑ)       ; U+1A51 TAI THAM LETTER EE
 ("G"  ?ᨫ)      ; U+1A2B TAI THAM LETTER LOW CHA
 ("h"  ?\u1a76) ; U+1A76 TAI THAM SIGN TONE-2
 ("H"  ?\u1a6b) ; U+1A6B TAI THAM VOWEL SIGN O
 ("j"  ?\u1a75) ; U+1A75 TAI THAM SIGN TONE-1
; ("J"  ?) ; U+
 ("k"  ?\u1a63) ; U+1A63 TAI THAM VOWEL SIGN AA
 ("K"  ?ᩇ)      ; U+1A47 TAI THAM LETTER HIGH SSA
 ("l"  ?ᩈ)      ; U+1A48 TAI THAM LETTER HIGH SA
 ("!l" ?\u1a5e) ; U+1A5E TAI THAM CONSONANT SIGN SA
 ("L"  ?ᩆ)      ; U+1A46 TAI THAM LETTER HIGH SHA
 ("!L" ?ᩔ)      ; U+1A54 TAI THAM LETTER GREAT SA
 (";"  ?ᩅ)      ; U+1A45;TAI THAM LETTER WA
 ("!;" ["ᩉ᩠ᩅ"])  ; U+1A49 U+1A60 U+1A45
 (":"  ?ᨪ)      ; U+1A2A TAI THAM LETTER LOW SA
 ("'"  ?ᨦ)      ; U+1A26;TAI THAM LETTER NGA
 ("!'" ["ᩉ᩠ᨦ"]) ; U+1A49 U+1A60 U+1A26
 ("@"  ?.)      ; U+002E FULL STOP
 ("!@" ?\u1a59) ; U+1A59 TAI THAM CONSONANT SIGN FINAL NGA
 ("#"  ?ᨢ)      ; U+1A22 TAI THAM LETTER HIGH KXA
 ("!#" ?\u1a58) ; U+1A58 TAI THAM SIGN MAI KANG LAI
 ("~"  ?ᨤ)      ; U+1A24 TAI THAM LETTER LOW KXA
 ("!~" ?\u200c) ; U+200C ZERO WIDTH NON-JOINER

 ("\\"  ?ᩀ)     ; U+1A40;TAI THAM LETTER HIGH YA
 ("|"  ?\u1a55) ; U+1A55;TAI THAM CONSONANT SIGN MEDIAL RA
 ("z"  ?ᨹ)      ; U+1A39 TAI THAM LETTER HIGH PHA
 ("Z"  ?()      ; U+0028 LEFT PARENTHESIS
 ("x"  ?ᨸ)      ; U+1A38;TAI THAM LETTER HIGH PA
 ("!x" ?\u1a6d) ; U+1A6D TAI THAM VOWEL SIGN OY
 ("X"  ?))      ; U+0029 RIGHT PARENTHESIS
 ("c"  ?\u1a6f) ; U+1A6F TAI THAM VOWEL SIGN AE
 ("!c" ?ᩓ)      ; U+1A53 TAI THAM LETTER LAE
 ("C"  ?ᨨ)      ; U+1A28 TAI THAM LETTER HIGH CHA
 ("!C" ?\u1a57) ; U+1A57 TAI THAM CONSONANT SIGN LA TANG LAI
 ("v"  ?ᩋ)      ; U+1A4B TAI THAM LETTER A
 ("!v" ?\u1a6c) ; U+1A6C TAI THAM VOWEL SIGN OA BELOW
 ("V"  ?ᩌ)      ; U+1A4C TAI THAM LETTER LOW HA
 ("!V" ?\u1a6d) ; U+1A6D TAI THAM VOWEL SIGN OY
 ("b"  ?\u1a65) ; U+1A65 TAI THAM VOWEL SIGN I
 ("!b" ?\u1a4d) ; U+1A4D TAI THAM LETTER I
; ("B"  ?) ; U+
 ("!B" ?\u1a7f) ; U+1A7F TAI THAM COMBINING CRYPTOGRAMMIC DOT
 ("n"  ?\u1a68) ; U+1A68 TAI THAM VOWEL SIGN UUE
 ("!n" ?\u1a78) ; U+1A78 TAI THAM SIGN KHUEN TONE-4
 ("N"  ?\u1a7a) ; U+1A7A TAI THAM SIGN RA HAAM
 ("!N" ?\u1a7c) ; U+1A7C TAI THAM SIGN KHUEN-LUE KARAN
 ("m"  ?ᨴ)      ; U+1A34 TAI THAM LETTER LOW TA
 ("!m" ?\u2060) ; U+2060 WORD JOINER
 ("M"  ??)      ; U+003F QUESTION MARK
 ("!M" ?\u1a79) ; U+1A79 TAI THAM SIGN KHUEN TONE-5
 (","  ?ᨾ)      ; U+1A3E TAI THAM LETTER MA
 ("!," ["ᩉ᩠ᨾ"]) ; U+1A49 U+1A60 U+1A3E
 ("<"  ?ᨰ)      ; U+1A30 TAI THAM LETTER LOW RATHA
 ("!<" ?\u1a5c) ; U+1A5C TAI THAM CONSONANT SIGN MA
 ("."  ?\u1a72) ; U+1A72 TAI THAM VOWEL SIGN THAM AI
 (">"  ?ᩊ)      ; U+1A4A TAI THAM LETTER LLA
 ("/"  ?ᨺ)      ; U+1A3A TAI THAM LETTER HIGH FA
 ("?"  ?ᩄ)      ; U+1A44 TAI THAM LETTER LUE

; 1AA0;TAI THAM SIGN WIANG
; 1AA1;TAI THAM SIGN WIANGWAAK
; 1AA2;TAI THAM SIGN SAWAN
; 1AA3;TAI THAM SIGN KEOW
; 1AA4;TAI THAM SIGN HOY
; 1AA5;TAI THAM SIGN DOKMAI
; 1AA6;TAI THAM SIGN REVERSED ROTATED RANA
; 1AAC;TAI THAM SIGN HANG
; 1AAD;TAI THAM SIGN CAANG

)
